=== modified file 'include/client/mir_toolkit/mir_client_library.h'
--- include/client/mir_toolkit/mir_client_library.h	2013-04-22 08:20:31 +0000
+++ include/client/mir_toolkit/mir_client_library.h	2013-04-24 05:22:20 +0000
@@ -267,6 +267,22 @@
  */
 MirSurfaceType mir_surface_get_type(MirSurface *surface);
 
+/**
+ * Change the state of a surface.
+ *   \param [in] surface  The surface to operate on
+ *   \param [in] state    The new state of the surface
+ *   \return              A wait handle that can be passed to mir_wait_for
+ */
+MirWaitHandle* mir_surface_set_state(MirSurface *surface,
+                                     MirSurfaceState state);
+
+/**
+ * Get the current state of a surface.
+ *   \param [in] surface  The surface to query
+ *   \return              The state of the surface
+ */
+MirSurfaceState mir_surface_get_state(MirSurface *surface);
+
 #ifdef __cplusplus
 }
 /**@}*/

=== added file 'include/server/mir/event_queue.h'
--- include/server/mir/event_queue.h	1970-01-01 00:00:00 +0000
+++ include/server/mir/event_queue.h	2013-04-16 05:46:28 +0000
@@ -0,0 +1,40 @@
+/*
+ * Copyright © 2013 Canonical Ltd.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3 as
+ * published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authored by: Daniel van Vugt <daniel.van.vugt@canonical.com>
+ */
+
+#ifndef MIR_EVENT_QUEUE_H_
+#define MIR_EVENT_QUEUE_H_
+
+#include "mir_toolkit/event.h"
+#include "mir/event_sink.h"
+#include <memory>
+
+namespace mir
+{
+class EventQueue : public EventSink
+{
+public:
+    void set_target(std::weak_ptr<EventSink> const& s);
+    void handle_event(MirEvent const& e) override;
+
+private:
+    std::weak_ptr<EventSink> target;
+};
+
+} // namespace mir
+
+#endif

=== modified file 'include/server/mir/frontend/protobuf_ipc_factory.h'
--- include/server/mir/frontend/protobuf_ipc_factory.h	2013-04-16 09:08:29 +0000
+++ include/server/mir/frontend/protobuf_ipc_factory.h	2013-04-24 05:22:20 +0000
@@ -23,6 +23,8 @@
 
 namespace mir
 {
+class EventSink;
+
 namespace protobuf
 {
 class DisplayServer;
@@ -35,7 +37,8 @@
 class ProtobufIpcFactory
 {
 public:
-    virtual std::shared_ptr<protobuf::DisplayServer> make_ipc_server() = 0;
+    virtual std::shared_ptr<protobuf::DisplayServer> make_ipc_server(
+        std::shared_ptr<EventSink> const& sink) = 0;
     virtual std::shared_ptr<ResourceCache> resource_cache() = 0;
     virtual std::shared_ptr<MessageProcessorReport> report() = 0;
 

=== modified file 'include/server/mir/frontend/session.h'
--- include/server/mir/frontend/session.h	2013-04-19 09:21:06 +0000
+++ include/server/mir/frontend/session.h	2013-04-24 05:22:20 +0000
@@ -29,6 +29,7 @@
 
 namespace mir
 {
+class EventSink;
 
 namespace frontend
 {
@@ -51,6 +52,8 @@
 
     virtual int configure_surface(SurfaceId id, MirSurfaceAttrib attrib, int value) = 0;
 
+    virtual void set_event_sink(std::shared_ptr<EventSink> const&) = 0;
+
 protected:
     Session() = default;
     Session(Session const&) = delete;

=== modified file 'include/server/mir/frontend/session_mediator.h'
--- include/server/mir/frontend/session_mediator.h	2013-04-16 09:08:29 +0000
+++ include/server/mir/frontend/session_mediator.h	2013-04-24 05:22:20 +0000
@@ -27,6 +27,8 @@
 
 namespace mir
 {
+class EventSink;
+
 namespace graphics
 {
 class Platform;
@@ -60,6 +62,7 @@
         std::shared_ptr<graphics::ViewableArea> const& viewable_area,
         std::shared_ptr<compositor::GraphicBufferAllocator> const& buffer_allocator,
         std::shared_ptr<SessionMediatorReport> const& report,
+        std::shared_ptr<EventSink> const& event_sink,
         std::shared_ptr<ResourceCache> const& resource_cache);
 
     /* Platform independent requests */
@@ -116,6 +119,7 @@
     std::shared_ptr<compositor::GraphicBufferAllocator> const buffer_allocator;
 
     std::shared_ptr<SessionMediatorReport> const report;
+    std::shared_ptr<EventSink> const event_sink;
     std::shared_ptr<ResourceCache> const resource_cache;
     std::shared_ptr<ClientBufferTracker> const client_tracker;
 

=== modified file 'include/server/mir/shell/application_session.h'
--- include/server/mir/shell/application_session.h	2013-04-18 17:22:34 +0000
+++ include/server/mir/shell/application_session.h	2013-04-24 05:22:20 +0000
@@ -25,6 +25,7 @@
 
 namespace mir
 {
+class EventSink;
 
 namespace shell
 {
@@ -54,6 +55,8 @@
 
     int configure_surface(frontend::SurfaceId id, MirSurfaceAttrib attrib, int value);
 
+    void set_event_sink(std::shared_ptr<mir::EventSink> const& sink);
+
 protected:
     ApplicationSession(ApplicationSession const&) = delete;
     ApplicationSession& operator=(ApplicationSession const&) = delete;
@@ -71,6 +74,8 @@
     Surfaces::const_iterator checked_find(frontend::SurfaceId id) const;
     std::mutex mutable surfaces_mutex;
     Surfaces surfaces;
+
+    std::shared_ptr<EventSink> event_sink;
 };
 
 }

=== modified file 'include/server/mir/shell/surface.h'
--- include/server/mir/shell/surface.h	2013-04-16 17:42:26 +0000
+++ include/server/mir/shell/surface.h	2013-04-24 05:22:20 +0000
@@ -21,6 +21,7 @@
 #define MIR_SHELL_SURFACE_H_
 
 #include "mir/frontend/surface.h"
+#include "mir/frontend/surface_id.h"
 #include "mir/surfaces/surface.h"
 #include "mir/input/surface_target.h"
 
@@ -30,6 +31,9 @@
 
 namespace mir
 {
+
+class EventSink;
+
 namespace frontend
 {
 struct SurfaceCreationParameters;
@@ -76,15 +80,25 @@
 
     virtual int configure(MirSurfaceAttrib attrib, int value);
     virtual MirSurfaceType type() const;
+    virtual MirSurfaceState state() const;
+
+    void set_id(frontend::SurfaceId i);
+    void set_event_target(std::shared_ptr<EventSink> const& sink);
 
 private:
     bool set_type(MirSurfaceType t);  // Use configure() to make public changes
+    bool set_state(MirSurfaceState s);
+    void notify_change(MirSurfaceAttrib attrib, int value);
 
     std::shared_ptr<SurfaceBuilder> const builder;
     std::shared_ptr<mir::input::InputChannel> const input_channel;
     std::weak_ptr<mir::surfaces::Surface> const surface;
 
+    std::shared_ptr<EventSink> event_sink;
+    frontend::SurfaceId id;
+
     MirSurfaceType type_value;
+    MirSurfaceState state_value;
 };
 }
 }

=== added file 'include/shared/mir/event_sink.h'
--- include/shared/mir/event_sink.h	1970-01-01 00:00:00 +0000
+++ include/shared/mir/event_sink.h	2013-04-19 04:33:51 +0000
@@ -0,0 +1,43 @@
+/*
+ * Copyright © 2013 Canonical Ltd.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License version 3 as
+ * published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authored by: Daniel van Vugt <daniel.van.vugt@canonical.com>
+ */
+
+#ifndef MIR_EVENT_SINK_H_
+#define MIR_EVENT_SINK_H_
+
+#include "mir_toolkit/event.h"
+
+namespace mir
+{
+
+class EventSink
+{
+public:
+    virtual ~EventSink() = default;
+
+    virtual void handle_event(MirEvent const& e) = 0;
+
+protected:
+    EventSink() = default;
+    EventSink(EventSink const&) = delete;
+    EventSink& operator=(EventSink const&) = delete;
+};
+
+} // namespace mir
+
+#endif
+

=== modified file 'include/shared/mir_toolkit/common.h'
--- include/shared/mir_toolkit/common.h	2013-03-29 11:39:30 +0000
+++ include/shared/mir_toolkit/common.h	2013-04-24 05:22:20 +0000
@@ -34,6 +34,7 @@
 typedef enum MirSurfaceAttrib
 {
     mir_surface_attrib_type,
+    mir_surface_attrib_state,
     mir_surface_attrib_arraysize_
 } MirSurfaceAttrib;
 
@@ -48,7 +49,20 @@
     mir_surface_type_arraysize_
 } MirSurfaceType;
 
-/* TODO: Surface states here */
+typedef enum MirSurfaceState
+{
+    mir_surface_state_unknown,
+    mir_surface_state_restored,
+    mir_surface_state_minimized,
+    mir_surface_state_maximized,
+    mir_surface_state_vertmaximized,
+    /* mir_surface_state_semimaximized,
+       Omitted for now, since it's functionally a subset of vertmaximized and
+       differs only in the X coordinate. */
+    mir_surface_state_fullscreen,
+    mir_surface_state_arraysize_
+} MirSurfaceState;
+
 /**@}*/
 
 #endif

=== modified file 'include/shared/mir_toolkit/event.h'
--- include/shared/mir_toolkit/event.h	2013-04-24 05:18:38 +0000
+++ include/shared/mir_toolkit/event.h	2013-04-24 05:22:20 +0000
@@ -21,6 +21,7 @@
 
 #include <stddef.h>
 #include <stdint.h>
+#include "mir_toolkit/common.h"
 
 #ifdef __cplusplus
 /**
@@ -37,7 +38,8 @@
 typedef enum
 {
     mir_event_type_key,
-    mir_event_type_motion
+    mir_event_type_motion,
+    mir_event_type_surface
 } MirEventType;
 
 enum {
@@ -111,11 +113,21 @@
     } pointer_coordinates[MIR_INPUT_EVENT_MAX_POINTER_COUNT];
 } MirMotionEvent;
 
+typedef struct
+{
+    MirEventType type;
+
+    int id;
+    MirSurfaceAttrib attrib;
+    int value;
+} MirSurfaceEvent;
+
 typedef union
 {
-    MirEventType   type;
-    MirKeyEvent    key;
-    MirMotionEvent motion;
+    MirEventType    type;
+    MirKeyEvent     key;
+    MirMotionEvent  motion;
+    MirSurfaceEvent surface;
 } MirEvent;
 
 #ifdef __cplusplus

=== modified file 'include/test/mir_test_doubles/mock_session.h'
--- include/test/mir_test_doubles/mock_session.h	2013-04-16 17:42:26 +0000
+++ include/test/mir_test_doubles/mock_session.h	2013-04-24 05:22:20 +0000
@@ -43,6 +43,8 @@
     MOCK_METHOD0(show, void());
 
     MOCK_METHOD3(configure_surface, int(frontend::SurfaceId, MirSurfaceAttrib, int));
+
+    MOCK_METHOD1(set_event_sink, void(std::shared_ptr<mir::EventSink> const&));
 };
 
 }

=== modified file 'include/test/mir_test_doubles/stub_ipc_factory.h'
--- include/test/mir_test_doubles/stub_ipc_factory.h	2013-04-16 09:08:29 +0000
+++ include/test/mir_test_doubles/stub_ipc_factory.h	2013-04-24 05:22:20 +0000
@@ -41,7 +41,8 @@
     {
     }
 
-    std::shared_ptr<protobuf::DisplayServer> make_ipc_server()
+    std::shared_ptr<protobuf::DisplayServer> make_ipc_server(
+        std::shared_ptr<EventSink> const&)
     {
         return server;
     }

=== modified file 'include/test/mir_test_doubles/stub_session.h'
--- include/test/mir_test_doubles/stub_session.h	2013-04-16 17:42:26 +0000
+++ include/test/mir_test_doubles/stub_session.h	2013-04-24 05:22:20 +0000
@@ -58,6 +58,9 @@
     {
         return 0;
     }
+    void set_event_sink(std::shared_ptr<EventSink> const&)
+    {
+    }
 };
 
 }

=== modified file 'src/client/mir_client_library.cpp'
--- src/client/mir_client_library.cpp	2013-04-22 08:20:31 +0000
+++ src/client/mir_client_library.cpp	2013-04-24 05:22:20 +0000
@@ -26,7 +26,7 @@
 #include "native_client_platform_factory.h"
 #include "egl_native_display_container.h"
 #include "mir_logger.h"
-#include "make_rpc_channel.h"
+#include "mir_socket_rpc_channel.h"
 
 #include <set>
 #include <unordered_set>
@@ -60,10 +60,12 @@
         auto log = std::make_shared<mcl::ConsoleLogger>();
         auto client_platform_factory = std::make_shared<mcl::NativeClientPlatformFactory>();
 
-        MirConnection* connection = new MirConnection(
-            mcl::make_rpc_channel(sock, log),
-            log,
-            client_platform_factory);
+        auto rpc = std::make_shared<mcl::MirSocketRpcChannel>(sock, log);
+
+        MirConnection* connection = new MirConnection(rpc, log,
+                                                      client_platform_factory);
+        
+        rpc->set_event_handler(connection);
 
         return connection->connect(name, callback, context);
     }
@@ -256,10 +258,12 @@
     auto log = std::make_shared<mcl::ConsoleLogger>();
     auto client_platform_factory = std::make_shared<mcl::NativeClientPlatformFactory>();
 
-    MirConnection* connection = new MirConnection(
-        mcl::make_rpc_channel(server, log),
-        log,
-        client_platform_factory);
+    auto rpc = std::make_shared<mcl::MirSocketRpcChannel>(server, log);
+
+    MirConnection* connection = new MirConnection(rpc, log,
+                                                  client_platform_factory);
+
+    rpc->set_event_handler(connection);
 
     return connection->connect(lightdm_id, app_name, callback, client_context);
 }
@@ -301,3 +305,29 @@
 
     return type;
 }
+
+MirWaitHandle* mir_surface_set_state(MirSurface *surf, MirSurfaceState state)
+{
+    return surf ? surf->configure(mir_surface_attrib_state, state) : NULL;
+}
+
+MirSurfaceState mir_surface_get_state(MirSurface *surf)
+{
+    MirSurfaceState state = mir_surface_state_unknown;
+
+    if (surf)
+    {
+        int s = surf->attrib(mir_surface_attrib_state);
+
+        if (s == mir_surface_state_unknown)
+        {
+            surf->configure(mir_surface_attrib_state,
+                            mir_surface_state_unknown)->wait_for_result();
+            s = surf->attrib(mir_surface_attrib_state);
+        }
+
+        state = static_cast<MirSurfaceState>(s);
+    }
+
+    return state;
+}

=== modified file 'src/client/mir_connection.cpp'
--- src/client/mir_connection.cpp	2013-04-11 22:47:09 +0000
+++ src/client/mir_connection.cpp	2013-04-24 05:22:20 +0000
@@ -59,7 +59,7 @@
     connect_result.set_error("connect not called");
 }
 
-MirConnection::~MirConnection()
+MirConnection::~MirConnection() noexcept
 {
     std::lock_guard<std::mutex> lock(connection_guard);
     valid_connections.erase(this);
@@ -120,6 +120,8 @@
 
     SurfaceRelease surf_release{surface, new_wait_handle, callback, context};
 
+    valid_surfaces.erase(surface->id());
+
     mir::protobuf::SurfaceId message;
     message.set_value(surface->id());
     server.release_surface(0, &message, &void_response,
@@ -312,3 +314,44 @@
 {
     return *native_display;
 }
+
+void MirConnection::on_surface_created(int id, MirSurface* surface)
+{
+    valid_surfaces[id] = surface;
+}
+
+void MirConnection::handle_event(MirEvent const& e)
+{
+    switch (e.type)
+    {
+    case mir_event_type_surface:
+        {
+            int id = e.surface.id;
+            SurfaceMap::iterator it = valid_surfaces.find(id);
+            if (it != valid_surfaces.end())
+            {
+                MirSurface *surface = it->second;
+                surface->handle_event(e);
+            }
+            else
+            {
+                log->error() << __PRETTY_FUNCTION__
+                             << ": mir_event_type_surface "
+                             << "received for non-existent surface ID "
+                             << id
+                             << ".\n";
+            }
+        }
+        break;
+    default:
+        // Don't worry. This function only gets called for events from the
+        // RPC channel (not input). So you will never see this error unless
+        // you make a mistake.
+
+        log->error() << __PRETTY_FUNCTION__
+                     << ": Unsupported event type " 
+                     << e.type
+                     << " received from server.\n";
+        break;
+    }
+}

=== modified file 'src/client/mir_connection.h'
--- src/client/mir_connection.h	2013-04-11 22:47:09 +0000
+++ src/client/mir_connection.h	2013-04-24 05:22:20 +0000
@@ -21,6 +21,7 @@
 #include <string>
 #include <memory>
 #include <unordered_set>
+#include <unordered_map>
 
 #include <mutex>
 
@@ -33,6 +34,7 @@
 #include "client_context.h"
 
 #include "mir_wait_handle.h"
+#include "mir/event_sink.h"
 
 namespace mir
 {
@@ -49,7 +51,8 @@
 }
 }
 
-struct MirConnection : public mir::client::ClientContext
+struct MirConnection : public mir::client::ClientContext,
+                       public mir::EventSink
 {
 public:
     MirConnection();
@@ -57,7 +60,7 @@
     MirConnection(std::shared_ptr<google::protobuf::RpcChannel> const& channel,
                   std::shared_ptr<mir::client::Logger> const & log,
                   std::shared_ptr<mir::client::ClientPlatformFactory> const& client_platform_factory);
-    ~MirConnection();
+    ~MirConnection() noexcept;
 
     MirConnection(MirConnection const &) = delete;
     MirConnection& operator=(MirConnection const &) = delete;
@@ -104,6 +107,9 @@
 
     EGLNativeDisplayType egl_native_display();
 
+    void on_surface_created(int id, MirSurface* surface);
+    void handle_event(MirEvent const&);
+
 private:
     std::shared_ptr<google::protobuf::RpcChannel> channel;
     mir::protobuf::DisplayServer::Stub server;
@@ -132,6 +138,9 @@
     static std::mutex connection_guard;
     static std::unordered_set<MirConnection*> valid_connections;
 
+    typedef std::unordered_map<int, MirSurface*> SurfaceMap;
+    SurfaceMap valid_surfaces;
+
     struct SurfaceRelease;
 
     void done_disconnect();

=== modified file 'src/client/mir_socket_rpc_channel.cpp'
--- src/client/mir_socket_rpc_channel.cpp	2013-04-11 22:47:09 +0000
+++ src/client/mir_socket_rpc_channel.cpp	2013-04-24 05:22:20 +0000
@@ -27,6 +27,7 @@
 #include "ancillary.h"
 
 #include <boost/bind.hpp>
+#include <cstring>
 
 namespace mcl = mir::client;
 namespace mcld = mir::client::detail;
@@ -36,8 +37,15 @@
 {
 }
 
-mcl::MirSocketRpcChannel::MirSocketRpcChannel(std::string const& endpoint, std::shared_ptr<Logger> const& log) :
-    log(log), pending_calls(log), work(io_service), endpoint(endpoint), socket(io_service)
+mcl::MirSocketRpcChannel::MirSocketRpcChannel(
+    std::string const& endpoint,
+    std::shared_ptr<Logger> const& log) :
+    log(log),
+    pending_calls(log),
+    work(io_service),
+    endpoint(endpoint),
+    socket(io_service),
+    event_handler(nullptr)
 {
     socket.connect(endpoint);
 
@@ -245,7 +253,14 @@
 
         log->debug() << __PRETTY_FUNCTION__ << " result.id():" << result.id() << std::endl;
 
-        pending_calls.complete_response(result);
+        if (!result.has_id())  // It's an event sequence
+        {
+            process_event_sequence(result);
+        }
+        else
+        {
+            pending_calls.complete_response(result);
+        }
     }
     catch (std::exception const& x)
     {
@@ -254,6 +269,46 @@
     }
 }
 
+void mcl::MirSocketRpcChannel::process_event_sequence(
+    mir::protobuf::wire::Result const& result)
+{
+    if (!event_handler)
+        return;
+
+    mir::protobuf::EventSequence seq;
+    if (seq.ParseFromString(result.response()))
+    {
+        int const nevents = seq.event_size();
+        for (int i = 0; i < nevents; i++)
+        {
+            mir::protobuf::Event const& event = seq.event(i);
+            if (event.has_raw())
+            {
+                std::string const& raw_event = event.raw();
+
+                // In future, events might be compressed where possible.
+                // But that's a job for later...
+                if (raw_event.size() == sizeof(MirEvent))
+                {
+                    MirEvent e;
+
+                    // Make a copy to ensure integer fields get correct memory
+                    // alignment, which is critical on many non-x86
+                    // architectures.
+                    memcpy(&e, raw_event.data(), sizeof e);
+                    event_handler->handle_event(e);
+                }
+                else
+                {
+                    log->error() << __PRETTY_FUNCTION__
+                                 << " Received MirEvent of an unexpected size."
+                                 << std::endl;
+                }
+            }
+        }
+    } // else protobuf will log an error
+}
+
 size_t mcl::MirSocketRpcChannel::read_message_header()
 {
     const size_t body_size = (header_bytes[0] << 8) + header_bytes[1];
@@ -275,3 +330,13 @@
     result.ParseFromIstream(&in);
     return result;
 }
+
+void mcl::MirSocketRpcChannel::set_event_handler(mir::EventSink *sink)
+{
+    /*
+     * Yes, these have to be regular pointers. Because ownership of the object
+     * (which is actually a MirConnection) is the responsibility of the calling
+     * client. So out of our control.
+     */
+    event_handler = sink;
+}

=== modified file 'src/client/mir_socket_rpc_channel.h'
--- src/client/mir_socket_rpc_channel.h	2013-04-11 22:47:09 +0000
+++ src/client/mir_socket_rpc_channel.h	2013-04-24 05:22:20 +0000
@@ -22,6 +22,7 @@
 
 #include "mir_basic_rpc_channel.h"
 #include "mir_logger.h"
+#include "mir/event_sink.h"
 
 #include <boost/asio.hpp>
 
@@ -51,6 +52,8 @@
     MirSocketRpcChannel(const std::string& endpoint, const std::shared_ptr<Logger>& log);
     ~MirSocketRpcChannel();
 
+    void set_event_handler(EventSink *sink);
+
 private:
     virtual void CallMethod(const google::protobuf::MethodDescriptor* method, google::protobuf::RpcController*,
         const google::protobuf::Message* parameters, google::protobuf::Message* response,
@@ -72,10 +75,13 @@
     void on_header_read(const boost::system::error_code& error);
 
     void read_message();
+    void process_event_sequence(mir::protobuf::wire::Result const& result);
 
     size_t read_message_header();
 
     mir::protobuf::wire::Result read_message_body(const size_t body_size);
+
+    EventSink *event_handler;
 };
 
 }

=== modified file 'src/client/mir_surface.cpp'
--- src/client/mir_surface.cpp	2013-04-11 22:47:09 +0000
+++ src/client/mir_surface.cpp	2013-04-24 05:22:20 +0000
@@ -59,6 +59,7 @@
     for (int i = 0; i < mir_surface_attrib_arraysize_; i++)
         attrib_cache[i] = -1;
     attrib_cache[mir_surface_attrib_type] = mir_surface_type_normal;
+    attrib_cache[mir_surface_attrib_state] = mir_surface_state_unknown;
 }
 
 MirSurface::~MirSurface()
@@ -176,6 +177,7 @@
     auto platform = connection->get_client_platform();
     accelerated_window = platform->create_egl_native_window(this);
 
+    connection->on_surface_created(id(), this);
     callback(this, context);
     
     create_wait_handle.result_received();
@@ -261,14 +263,16 @@
         configure_result.surfaceid().value() == surface.id().value() &&
         configure_result.has_attrib())
     {
-        switch (configure_result.attrib())
+        int a = configure_result.attrib();
+
+        switch (a)
         {
         case mir_surface_attrib_type:
+        case mir_surface_attrib_state:
             if (configure_result.has_ivalue())
-            {
-                int t = configure_result.ivalue();
-                attrib_cache[mir_surface_attrib_type] = t;
-            } // else error is probably set due to an unsupported attrib/value
+                attrib_cache[a] = configure_result.ivalue();
+            else
+                assert(configure_result.has_error());
             break;
         default:
             assert(false);
@@ -307,3 +311,16 @@
         }
     }
 }
+
+void MirSurface::handle_event(MirEvent const& e)
+{
+    if (e.type == mir_event_type_surface)
+    {
+        MirSurfaceAttrib a = e.surface.attrib;
+        if (a < mir_surface_attrib_arraysize_)
+            attrib_cache[a] = e.surface.value;
+    }
+
+    if (handle_event_callback)
+        handle_event_callback(&e);
+}

=== modified file 'src/client/mir_surface.h'
--- src/client/mir_surface.h	2013-04-11 22:47:09 +0000
+++ src/client/mir_surface.h	2013-04-24 05:22:20 +0000
@@ -85,6 +85,7 @@
     int attrib(MirSurfaceAttrib a) const;
 
     void set_event_handler(MirEventDelegate const* delegate);
+    void handle_event(MirEvent const& e);
 
 private:
     void on_configured();

=== modified file 'src/server/CMakeLists.txt'
--- src/server/CMakeLists.txt	2013-04-10 16:23:54 +0000
+++ src/server/CMakeLists.txt	2013-04-24 05:22:20 +0000
@@ -26,6 +26,7 @@
   display_server.cpp
   default_server_configuration.cpp
   asio_main_loop.cpp
+  event_queue.cpp
 )
 
 set(MIRSERVER_LINKAGE SHARED)

=== modified file 'src/server/default_server_configuration.cpp'
--- src/server/default_server_configuration.cpp	2013-04-24 05:18:38 +0000
+++ src/server/default_server_configuration.cpp	2013-04-24 05:22:20 +0000
@@ -108,7 +108,8 @@
     std::shared_ptr<mg::ViewableArea> const graphics_display;
     std::shared_ptr<mc::GraphicBufferAllocator> const buffer_allocator;
 
-    virtual std::shared_ptr<mir::protobuf::DisplayServer> make_ipc_server()
+    virtual std::shared_ptr<mir::protobuf::DisplayServer> make_ipc_server(
+        std::shared_ptr<mir::EventSink> const& sink)
     {
         return std::make_shared<mf::SessionMediator>(
             shell,
@@ -116,6 +117,7 @@
             graphics_display,
             buffer_allocator,
             sm_report,
+            sink,
             resource_cache());
     }
 

=== added file 'src/server/event_queue.cpp'
--- src/server/event_queue.cpp	1970-01-01 00:00:00 +0000
+++ src/server/event_queue.cpp	2013-04-24 03:44:27 +0000
@@ -0,0 +1,37 @@
+/*
+ * Copyright © 2013 Canonical Ltd.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3 as
+ * published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authored by: Daniel van Vugt <daniel.van.vugt@canonical.com>
+ */
+
+#include "mir/event_queue.h"
+#include "mir/event_sink.h"
+
+using namespace mir;
+
+void EventQueue::set_target(std::weak_ptr<EventSink> const& s)
+{
+    target = s;
+}
+
+void EventQueue::handle_event(MirEvent const& e)
+{
+    // In future, we might put e on a queue and wait for some background
+    // thread to push it through to target. But that's not required right now.
+
+    std::shared_ptr<EventSink> p = target.lock();
+    if (p)
+        p->handle_event(e);
+}

=== modified file 'src/server/frontend/protobuf_message_processor.cpp'
--- src/server/frontend/protobuf_message_processor.cpp	2013-04-24 05:18:38 +0000
+++ src/server/frontend/protobuf_message_processor.cpp	2013-04-24 05:22:20 +0000
@@ -16,6 +16,7 @@
  * Authored by: Alan Griffiths <alan@octopull.co.uk>
  */
 
+#include "mir_toolkit/event.h"
 #include "protobuf_message_processor.h"
 #include "mir/frontend/message_processor_report.h"
 #include "mir/frontend/resource_cache.h"
@@ -135,6 +136,32 @@
     sender->send(buffer);
 }
 
+void mfd::ProtobufMessageProcessor::send_event(MirEvent const& e)
+{
+    mir::protobuf::EventSequence seq;
+    mir::protobuf::Event *ev = seq.add_event();
+    ev->set_raw(&e, sizeof(MirEvent));
+
+    std::string buffer;
+    seq.SerializeToString(&buffer);
+
+    mir::protobuf::wire::Result result;
+    result.set_response(buffer);
+
+    result.SerializeToString(&buffer);
+
+    sender->send(buffer);
+}
+
+void mfd::ProtobufMessageProcessor::handle_event(MirEvent const& e)
+{
+    // Limit the types of events we wish to send over protobuf, for now.
+    if (e.type == mir_event_type_surface)
+    {
+        send_event(e);
+    }
+}
+
 bool mfd::ProtobufMessageProcessor::dispatch(mir::protobuf::wire::Invocation const& invocation)
 {
     report->received_invocation(display_server.get(), invocation.id(), invocation.method_name());

=== modified file 'src/server/frontend/protobuf_message_processor.h'
--- src/server/frontend/protobuf_message_processor.h	2013-04-24 05:18:38 +0000
+++ src/server/frontend/protobuf_message_processor.h	2013-04-24 05:22:20 +0000
@@ -24,6 +24,7 @@
 
 #include "mir_protobuf.pb.h"
 #include "mir_protobuf_wire.pb.h"
+#include "mir/event_sink.h"
 
 #include <vector>
 #include <memory>
@@ -42,7 +43,8 @@
 namespace detail
 {
 
-struct ProtobufMessageProcessor : MessageProcessor
+struct ProtobufMessageProcessor : MessageProcessor,
+                                  public EventSink
 {
     ProtobufMessageProcessor(
         MessageSender* sender,
@@ -68,6 +70,10 @@
     // OTOH until we have a real requirement it is hard to see how best to generalise.
     void send_response(::google::protobuf::uint32 id, mir::protobuf::Surface* response);
 
+    void send_event(MirEvent const& e);
+
+    void handle_event(MirEvent const& e);
+
     template<class Response>
     std::vector<int32_t> extract_fds_from(Response* response);
 

=== modified file 'src/server/frontend/protobuf_socket_communicator.cpp'
--- src/server/frontend/protobuf_socket_communicator.cpp	2013-04-19 14:00:35 +0000
+++ src/server/frontend/protobuf_socket_communicator.cpp	2013-04-24 05:22:20 +0000
@@ -22,6 +22,8 @@
 
 #include "mir/frontend/protobuf_ipc_factory.h"
 #include "mir/protobuf/google_protobuf_guard.h"
+#include "mir/event_sink.h"
+#include "mir/event_queue.h"
 
 #include <boost/signals2.hpp>
 
@@ -49,6 +51,8 @@
 
 void mf::ProtobufSocketCommunicator::start_accept()
 {
+    std::shared_ptr<EventQueue> event_queue = std::make_shared<EventQueue>();
+
     auto const& socket_session = std::make_shared<mfd::SocketSession>(
         io_service,
         next_id(),
@@ -56,10 +60,11 @@
 
     auto session = std::make_shared<detail::ProtobufMessageProcessor>(
         socket_session.get(),
-        ipc_factory->make_ipc_server(),
+        ipc_factory->make_ipc_server(event_queue),
         ipc_factory->resource_cache(),
         ipc_factory->report());
 
+    event_queue->set_target(session);
     socket_session->set_processor(session);
 
     acceptor.async_accept(

=== modified file 'src/server/frontend/session_mediator.cpp'
--- src/server/frontend/session_mediator.cpp	2013-04-16 09:08:29 +0000
+++ src/server/frontend/session_mediator.cpp	2013-04-24 05:22:20 +0000
@@ -44,12 +44,14 @@
     std::shared_ptr<graphics::ViewableArea> const& viewable_area,
     std::shared_ptr<compositor::GraphicBufferAllocator> const& buffer_allocator,
     std::shared_ptr<SessionMediatorReport> const& report,
+    std::shared_ptr<EventSink> const& event_sink,
     std::shared_ptr<ResourceCache> const& resource_cache) :
     shell(shell),
     graphics_platform(graphics_platform),
     viewable_area(viewable_area),
     buffer_allocator(buffer_allocator),
     report(report),
+    event_sink(event_sink),
     resource_cache(resource_cache),
     client_tracker(std::make_shared<ClientBufferTracker>(frontend::client_buffer_cache_size))
 {
@@ -64,6 +66,7 @@
     report->session_connect_called(request->application_name());
 
     session = shell->open_session(request->application_name());
+    session->set_event_sink(event_sink);
 
     auto ipc_package = graphics_platform->get_ipc_package();
     auto platform = response->mutable_platform();

=== modified file 'src/server/shell/application_session.cpp'
--- src/server/shell/application_session.cpp	2013-04-18 17:22:34 +0000
+++ src/server/shell/application_session.cpp	2013-04-24 05:22:20 +0000
@@ -53,6 +53,12 @@
     }
 }
 
+void msh::ApplicationSession::set_event_sink(
+    std::shared_ptr<mir::EventSink> const& sink)
+{
+    event_sink = sink;
+}
+
 mf::SurfaceId msh::ApplicationSession::next_id()
 {
     return mf::SurfaceId(next_surface_id.fetch_add(1));
@@ -63,6 +69,9 @@
     auto surf = surface_factory->create_surface(params);
     auto const id = next_id();
 
+    surf->set_id(id);
+    surf->set_event_target(event_sink);
+
     std::unique_lock<std::mutex> lock(surfaces_mutex);
     surfaces[id] = surf;
     return id;

=== modified file 'src/server/shell/surface.cpp'
--- src/server/shell/surface.cpp	2013-04-16 17:42:26 +0000
+++ src/server/shell/surface.cpp	2013-04-24 05:22:20 +0000
@@ -19,10 +19,13 @@
 #include "mir/shell/surface.h"
 #include "mir/shell/surface_builder.h"
 #include "mir/input/input_channel.h"
+#include "mir_toolkit/event.h"
+#include "mir/event_sink.h"
 
 #include <boost/throw_exception.hpp>
 
 #include <stdexcept>
+#include <cstring>
 
 namespace msh = mir::shell;
 namespace mc = mir::compositor;
@@ -35,7 +38,9 @@
   : builder(builder),
     input_channel(input_channel),
     surface(builder->create_surface(params)),
-    type_value(mir_surface_type_normal)
+    id(0),
+    type_value(mir_surface_type_normal),
+    state_value(mir_surface_state_restored)
 {
 }
 
@@ -47,6 +52,16 @@
     }
 }
 
+void msh::Surface::set_id(mir::frontend::SurfaceId i)
+{
+    id = i;
+}
+
+void msh::Surface::set_event_target(std::shared_ptr<EventSink> const& sink)
+{
+    event_sink = sink;
+}
+
 void msh::Surface::hide()
 {
     if (auto const& s = surface.lock())
@@ -169,6 +184,12 @@
                                                    "type."));
         result = type();
         break;
+    case mir_surface_attrib_state:
+        if (value != mir_surface_state_unknown &&
+            !set_state(static_cast<MirSurfaceState>(value)))
+            BOOST_THROW_EXCEPTION(std::logic_error("Invalid surface state."));
+        result = state();
+        break;
     default:
         BOOST_THROW_EXCEPTION(std::logic_error("Invalid surface "
                                                "attribute."));
@@ -195,3 +216,44 @@
 
     return valid;
 }
+
+MirSurfaceState msh::Surface::state() const
+{
+    return state_value;
+}
+
+bool msh::Surface::set_state(MirSurfaceState s)
+{
+    bool valid = false;
+
+    if (s > mir_surface_state_unknown &&
+        s < mir_surface_state_arraysize_)
+    {
+        state_value = s;
+        valid = true;
+
+        notify_change(mir_surface_attrib_state, s);
+    }
+
+    return valid;
+}
+
+void msh::Surface::notify_change(MirSurfaceAttrib attrib, int value)
+{
+    if (event_sink)
+    {
+        MirEvent e;
+
+        // This memset is not really required. However it does avoid some
+        // harmless uninitialized memory reads that valgrind will complain
+        // about, due to gaps in MirEvent.
+        memset(&e, 0, sizeof e);
+
+        e.type = mir_event_type_surface;
+        e.surface.id = id.as_value();
+        e.surface.attrib = attrib;
+        e.surface.value = value;
+
+        event_sink->handle_event(e);
+    }
+}

=== modified file 'src/shared/protobuf/mir_protobuf.proto'
--- src/shared/protobuf/mir_protobuf.proto	2013-03-21 03:32:59 +0000
+++ src/shared/protobuf/mir_protobuf.proto	2013-04-24 05:22:20 +0000
@@ -122,6 +122,14 @@
   optional string error = 127;
 }
 
+message Event {
+  optional bytes raw = 1;  // MirEvent structure
+}
+
+message EventSequence {
+  repeated Event event = 1;
+}
+
 service DisplayServer {
   // Platform independent requests
   rpc connect(ConnectParameters) returns (Connection);

=== modified file 'src/shared/protobuf/mir_protobuf_wire.proto'
--- src/shared/protobuf/mir_protobuf_wire.proto	2012-09-14 12:03:52 +0000
+++ src/shared/protobuf/mir_protobuf_wire.proto	2013-04-24 05:22:20 +0000
@@ -7,6 +7,7 @@
 }
 
 message Result {
-  required uint32 id = 1;
+  // Invocation results have id and response. Events only have response.
+  optional uint32 id = 1;
   optional bytes response = 2;
 }

=== modified file 'tests/acceptance-tests/test_client_library.cpp'
--- tests/acceptance-tests/test_client_library.cpp	2013-04-17 07:31:40 +0000
+++ tests/acceptance-tests/test_client_library.cpp	2013-04-24 05:22:20 +0000
@@ -29,6 +29,7 @@
 
 #include <gtest/gtest.h>
 #include <chrono>
+#include <cstring>
 
 namespace mf = mir::frontend;
 namespace mc = mir::compositor;
@@ -279,6 +280,172 @@
     launch_client_process(client_config);
 }
 
+TEST_F(DefaultDisplayServerTestFixture, client_can_set_surface_state)
+{
+    struct ClientConfig : ClientConfigCommon
+    {
+        void exec()
+        {
+            connection = mir_connect_sync(mir_test_socket, __PRETTY_FUNCTION__);
+            ASSERT_TRUE(connection != NULL);
+            EXPECT_TRUE(mir_connection_is_valid(connection));
+            EXPECT_STREQ(mir_connection_get_error_message(connection), "");
+
+            MirSurfaceParameters const request_params =
+            {
+                __PRETTY_FUNCTION__,
+                640, 480,
+                mir_pixel_format_abgr_8888,
+                mir_buffer_usage_hardware
+            };
+
+            surface = mir_connection_create_surface_sync(connection,
+                                                         &request_params);
+            ASSERT_TRUE(surface != NULL);
+            EXPECT_TRUE(mir_surface_is_valid(surface));
+            EXPECT_STREQ(mir_surface_get_error_message(surface), "");
+
+            EXPECT_EQ(mir_surface_state_restored,
+                      mir_surface_get_state(surface));
+
+            mir_wait_for(mir_surface_set_state(surface,
+                                               mir_surface_state_fullscreen));
+            EXPECT_EQ(mir_surface_state_fullscreen,
+                      mir_surface_get_state(surface));
+
+            mir_wait_for(mir_surface_set_state(surface,
+                                           static_cast<MirSurfaceState>(999)));
+            EXPECT_EQ(mir_surface_state_fullscreen,
+                      mir_surface_get_state(surface));
+
+            mir_wait_for(mir_surface_set_state(surface,
+                                               mir_surface_state_minimized));
+            EXPECT_EQ(mir_surface_state_minimized,
+                      mir_surface_get_state(surface));
+
+            mir_wait_for(mir_surface_set_state(surface,
+                                           static_cast<MirSurfaceState>(888)));
+            EXPECT_EQ(mir_surface_state_minimized,
+                      mir_surface_get_state(surface));
+
+            // Stress-test synchronization logic with some flooding
+            for (int i = 0; i < 100; i++)
+            {
+                mir_surface_set_state(surface, mir_surface_state_maximized);
+                mir_surface_set_state(surface, mir_surface_state_restored);
+                mir_wait_for(mir_surface_set_state(surface,
+                                                mir_surface_state_fullscreen));
+                ASSERT_EQ(mir_surface_state_fullscreen,
+                          mir_surface_get_state(surface));
+            }
+
+            mir_surface_release_sync(surface);
+            mir_connection_release(connection);
+        }
+    } client_config;
+
+    launch_client_process(client_config);
+}
+
+TEST_F(DefaultDisplayServerTestFixture, client_receives_surface_state_events)
+{
+    struct ClientConfig : ClientConfigCommon
+    {
+        static void event_callback(MirSurface* surface, MirEvent const* event,
+                                   void* ctx)
+        {
+            ClientConfig* self = static_cast<ClientConfig*>(ctx);
+            self->last_event = *event;
+            self->last_event_surface = surface;
+        }
+
+        void exec()
+        {
+            connection = mir_connect_sync(mir_test_socket, __PRETTY_FUNCTION__);
+            ASSERT_TRUE(connection != NULL);
+            ASSERT_TRUE(mir_connection_is_valid(connection));
+
+            MirSurfaceParameters const request_params =
+            {
+                __PRETTY_FUNCTION__,
+                640, 480,
+                mir_pixel_format_abgr_8888,
+                mir_buffer_usage_hardware
+            };
+
+            memset(&last_event, 0, sizeof last_event);
+            last_event_surface = nullptr;
+
+            MirEventDelegate delegate{&event_callback, this};
+            MirSurface* other_surface =
+                mir_connection_create_surface_sync(connection, &request_params);
+            ASSERT_TRUE(other_surface != NULL);
+            ASSERT_TRUE(mir_surface_is_valid(other_surface));
+            mir_surface_set_event_handler(other_surface, nullptr);
+
+            surface =
+                mir_connection_create_surface_sync(connection, &request_params);
+            ASSERT_TRUE(surface != NULL);
+            ASSERT_TRUE(mir_surface_is_valid(surface));
+
+            mir_surface_set_event_handler(surface, &delegate);
+
+            int surface_id = mir_surface_get_id(surface);
+
+            mir_wait_for(mir_surface_set_state(surface,
+                                               mir_surface_state_fullscreen));
+            mir_wait_for(mir_surface_set_state(other_surface,
+                                               mir_surface_state_minimized));
+            EXPECT_EQ(surface, last_event_surface);
+            EXPECT_EQ(mir_event_type_surface, last_event.type);
+            EXPECT_EQ(surface_id, last_event.surface.id);
+            EXPECT_EQ(mir_surface_attrib_state, last_event.surface.attrib);
+            EXPECT_EQ(mir_surface_state_fullscreen, last_event.surface.value);
+
+            mir_wait_for(mir_surface_set_state(surface,
+                                           static_cast<MirSurfaceState>(999)));
+            EXPECT_EQ(surface, last_event_surface);
+            EXPECT_EQ(mir_event_type_surface, last_event.type);
+            EXPECT_EQ(surface_id, last_event.surface.id);
+            EXPECT_EQ(mir_surface_attrib_state, last_event.surface.attrib);
+            EXPECT_EQ(mir_surface_state_fullscreen, last_event.surface.value);
+
+            memset(&last_event, 0, sizeof last_event);
+            last_event_surface = nullptr;
+
+            mir_wait_for(mir_surface_set_state(surface,
+                                               mir_surface_state_minimized));
+            EXPECT_EQ(surface, last_event_surface);
+            EXPECT_EQ(mir_event_type_surface, last_event.type);
+            EXPECT_EQ(surface_id, last_event.surface.id);
+            EXPECT_EQ(mir_surface_attrib_state, last_event.surface.attrib);
+            EXPECT_EQ(mir_surface_state_minimized, last_event.surface.value);
+
+            memset(&last_event, 0, sizeof last_event);
+            last_event_surface = nullptr;
+
+            mir_wait_for(mir_surface_set_state(surface,
+                                           static_cast<MirSurfaceState>(777)));
+            mir_wait_for(mir_surface_set_state(other_surface,
+                                               mir_surface_state_maximized));
+            EXPECT_EQ(0, last_event_surface);
+            EXPECT_EQ(0, last_event.type);
+            EXPECT_EQ(0, last_event.surface.id);
+            EXPECT_EQ(0, last_event.surface.attrib);
+            EXPECT_EQ(0, last_event.surface.value);
+
+            mir_surface_release_sync(surface);
+            mir_surface_release_sync(other_surface);
+            mir_connection_release(connection);
+        }
+
+        MirEvent last_event;
+        MirSurface* last_event_surface;
+    } client_config;
+
+    launch_client_process(client_config);
+}
+
 TEST_F(DefaultDisplayServerTestFixture, client_library_creates_multiple_surfaces)
 {
     int const n_surfaces = 13;

=== modified file 'tests/unit-tests/CMakeLists.txt'
--- tests/unit-tests/CMakeLists.txt	2013-04-08 16:20:19 +0000
+++ tests/unit-tests/CMakeLists.txt	2013-04-24 05:22:20 +0000
@@ -20,7 +20,7 @@
 add_subdirectory(android_input/)
 add_subdirectory(surfaces/)
 add_subdirectory(draw/)
-
+add_subdirectory(event/)
 
 add_executable(unit-tests ${UNIT_TEST_SOURCES})
 uses_android_input(unit-tests)

=== modified file 'tests/unit-tests/client/test_client_mir_surface.cpp'
--- tests/unit-tests/client/test_client_mir_surface.cpp	2013-04-17 18:18:31 +0000
+++ tests/unit-tests/client/test_client_mir_surface.cpp	2013-04-24 05:22:20 +0000
@@ -540,3 +540,21 @@
     EXPECT_EQ(mir_surface_type_normal,
               surface->attrib(mir_surface_attrib_type));
 }
+
+TEST_F(MirClientSurfaceTest, default_surface_state)
+{
+    auto surface = std::make_shared<MirSurface> (connection.get(),
+                                                 *client_comm_channel,
+                                                 logger,
+                                                 mock_buffer_factory,
+                                                 input_platform,
+                                                 params,
+                                                 &empty_callback,
+                                                 nullptr);
+    surface->get_create_wait_handle()->wait_for_result();
+
+    // Test the default cached state value. It is always unknown until we
+    // get a real answer from the server.
+    EXPECT_EQ(mir_surface_state_unknown,
+              surface->attrib(mir_surface_attrib_state));
+}

=== added directory 'tests/unit-tests/event'
=== added file 'tests/unit-tests/event/CMakeLists.txt'
--- tests/unit-tests/event/CMakeLists.txt	1970-01-01 00:00:00 +0000
+++ tests/unit-tests/event/CMakeLists.txt	2013-04-09 08:23:32 +0000
@@ -0,0 +1,5 @@
+set (UNIT_TEST_SOURCES
+  ${UNIT_TEST_SOURCES}
+  ${CMAKE_CURRENT_SOURCE_DIR}/test_event_queue.cpp
+  PARENT_SCOPE
+)

=== added file 'tests/unit-tests/event/test_event_queue.cpp'
--- tests/unit-tests/event/test_event_queue.cpp	1970-01-01 00:00:00 +0000
+++ tests/unit-tests/event/test_event_queue.cpp	2013-04-22 03:37:28 +0000
@@ -0,0 +1,113 @@
+/*
+ * Copyright © 2013 Canonical Ltd.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 3 as
+ * published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Authored by: Daniel van Vugt <daniel.van.vugt@canonical.com>
+ */
+
+#include <gtest/gtest.h>
+#include "mir/event_queue.h"
+#include "mir/event_sink.h"
+#include <cstring>
+
+TEST(EventQueue, no_sink)
+{
+    mir::EventQueue q;
+    MirEvent e;
+
+    e.type = mir_event_type_key;
+    q.handle_event(e);
+
+    e.type = mir_event_type_motion;
+    q.handle_event(e);
+
+    e.type = mir_event_type_surface;
+    q.handle_event(e);
+}
+
+namespace
+{
+    class TestSink : public mir::EventSink
+    {
+    public:
+        TestSink()
+        {
+            memset(&last_event, 0, sizeof last_event);
+        }
+
+        void handle_event(MirEvent const& e)
+        {
+            last_event = e;
+        }
+
+        MirEvent const& last_event_handled() const
+        {
+            return last_event;
+        }
+
+    private:
+        MirEvent last_event;
+    };
+}
+
+TEST(EventQueue, events_get_handled)
+{
+    mir::EventQueue q;
+    std::shared_ptr<TestSink> s(new TestSink);
+
+    q.set_target(s);
+
+    MirEvent e;
+    memset(&e, 0, sizeof e);
+
+    e.type = mir_event_type_key;
+    q.handle_event(e);
+    EXPECT_EQ(mir_event_type_key, s->last_event_handled().type);
+
+    e.type = mir_event_type_motion;
+    q.handle_event(e);
+    EXPECT_EQ(mir_event_type_motion, s->last_event_handled().type);
+
+    e.type = mir_event_type_surface;
+    q.handle_event(e);
+    EXPECT_EQ(mir_event_type_surface, s->last_event_handled().type);
+}
+
+TEST(EventQueue, sink_is_changeable)
+{
+    mir::EventQueue q;
+    std::shared_ptr<TestSink> a(new TestSink);
+    std::shared_ptr<TestSink> b(new TestSink);
+
+    q.set_target(a);
+
+    MirEvent e;
+    memset(&e, 0, sizeof e);
+
+    e.type = mir_event_type_motion;
+    q.handle_event(e);
+    EXPECT_EQ(mir_event_type_motion, a->last_event_handled().type);
+
+    q.set_target(b);
+
+    e.type = mir_event_type_surface;
+    q.handle_event(e);
+    EXPECT_EQ(mir_event_type_surface, b->last_event_handled().type);
+    EXPECT_EQ(mir_event_type_motion, a->last_event_handled().type);
+
+    e.type = mir_event_type_key;
+    q.handle_event(e);
+    EXPECT_EQ(mir_event_type_key, b->last_event_handled().type);
+    EXPECT_EQ(mir_event_type_motion, a->last_event_handled().type);
+}

=== modified file 'tests/unit-tests/frontend/test_session_mediator.cpp'
--- tests/unit-tests/frontend/test_session_mediator.cpp	2013-04-11 02:10:40 +0000
+++ tests/unit-tests/frontend/test_session_mediator.cpp	2013-04-24 05:22:20 +0000
@@ -34,7 +34,7 @@
 #include "mir_test_doubles/stub_session.h"
 #include "mir_test_doubles/stub_surface_builder.h"
 #include "mir_test/fake_shared.h"
-
+#include "mir/event_queue.h"
 #include "mir/shell/surface.h"
 
 #include <gtest/gtest.h>
@@ -141,7 +141,9 @@
           report{std::make_shared<mf::NullSessionMediatorReport>()},
           resource_cache{std::make_shared<mf::ResourceCache>()},
           mediator{shell, graphics_platform, graphics_display,
-                   buffer_allocator, report, resource_cache},
+                   buffer_allocator, report, 
+                   std::make_shared<mir::EventQueue>(),
+                   resource_cache},
           stubbed_session{std::make_shared<StubbedSession>()},
           null_callback{google::protobuf::NewPermanentCallback(google::protobuf::DoNothing)}
     {

=== modified file 'tests/unit-tests/frontend/test_session_mediator_android.cpp'
--- tests/unit-tests/frontend/test_session_mediator_android.cpp	2013-04-10 16:40:04 +0000
+++ tests/unit-tests/frontend/test_session_mediator_android.cpp	2013-04-24 05:22:20 +0000
@@ -31,6 +31,8 @@
 #include "mir_test_doubles/mock_session.h"
 #include "mir_test_doubles/stub_shell.h"
 
+#include "mir/event_queue.h"
+
 #include <gtest/gtest.h>
 
 #include <stdexcept>
@@ -97,7 +99,9 @@
           report{std::make_shared<mf::NullSessionMediatorReport>()},
           resource_cache{std::make_shared<mf::ResourceCache>()},
           mediator{shell, graphics_platform, graphics_display,
-                   buffer_allocator, report, resource_cache},
+                   buffer_allocator, report,
+                   std::make_shared<mir::EventQueue>(),
+                   resource_cache},
           null_callback{google::protobuf::NewPermanentCallback(google::protobuf::DoNothing)}
     {
     }

=== modified file 'tests/unit-tests/frontend/test_session_mediator_gbm.cpp'
--- tests/unit-tests/frontend/test_session_mediator_gbm.cpp	2013-04-10 16:40:04 +0000
+++ tests/unit-tests/frontend/test_session_mediator_gbm.cpp	2013-04-24 05:22:20 +0000
@@ -27,6 +27,7 @@
 #include "mir/graphics/drm_authenticator.h"
 #include "mir/graphics/platform.h"
 #include "mir/graphics/platform_ipc_package.h"
+#include "mir/event_queue.h"
 
 #include <boost/exception/errinfo_errno.hpp>
 #include <boost/throw_exception.hpp>
@@ -102,7 +103,9 @@
           report{std::make_shared<mf::NullSessionMediatorReport>()},
           resource_cache{std::make_shared<mf::ResourceCache>()},
           mediator{shell, mock_platform, graphics_display,
-                   buffer_allocator, report, resource_cache},
+                   buffer_allocator, report,
+                   std::make_shared<mir::EventQueue>(),
+                   resource_cache},
           null_callback{google::protobuf::NewPermanentCallback(google::protobuf::DoNothing)}
     {
     }

=== modified file 'tests/unit-tests/shell/test_single_visibility_focus_mechanism.cpp'
--- tests/unit-tests/shell/test_single_visibility_focus_mechanism.cpp	2013-04-19 17:44:09 +0000
+++ tests/unit-tests/shell/test_single_visibility_focus_mechanism.cpp	2013-04-24 05:22:20 +0000
@@ -57,6 +57,7 @@
     MOCK_METHOD0(show, void());
 
     MOCK_METHOD3(configure_surface, int(mf::SurfaceId, MirSurfaceAttrib, int));
+    MOCK_METHOD1(set_event_sink, void(std::shared_ptr<mir::EventSink> const&));
 };
 
 TEST(SingleVisibilityFocusMechanism, mechanism_sets_visibility)

=== modified file 'tests/unit-tests/shell/test_surface.cpp'
--- tests/unit-tests/shell/test_surface.cpp	2013-04-16 17:42:26 +0000
+++ tests/unit-tests/shell/test_surface.cpp	2013-04-24 05:22:20 +0000
@@ -415,3 +415,38 @@
                              mir_surface_type_freestyle));
     EXPECT_EQ(mir_surface_type_freestyle, surf.type());
 }
+
+TEST_F(ShellSurface, states)
+{
+    using namespace testing;
+
+    msh::Surface surf(
+            mt::fake_shared(surface_builder),
+            mf::a_surface(),
+            null_input_channel);
+
+    EXPECT_EQ(mir_surface_state_restored, surf.state());
+
+    EXPECT_EQ(mir_surface_state_vertmaximized,
+              surf.configure(mir_surface_attrib_state,
+                             mir_surface_state_vertmaximized));
+    EXPECT_EQ(mir_surface_state_vertmaximized, surf.state());
+
+    EXPECT_THROW({
+        surf.configure(mir_surface_attrib_state, 999);
+    }, std::logic_error);
+    EXPECT_THROW({
+        surf.configure(mir_surface_attrib_state, -1);
+    }, std::logic_error);
+    EXPECT_EQ(mir_surface_state_vertmaximized, surf.state());
+
+    EXPECT_EQ(mir_surface_state_minimized,
+              surf.configure(mir_surface_attrib_state,
+                             mir_surface_state_minimized));
+    EXPECT_EQ(mir_surface_state_minimized, surf.state());
+
+    EXPECT_EQ(mir_surface_state_fullscreen,
+              surf.configure(mir_surface_attrib_state,
+                             mir_surface_state_fullscreen));
+    EXPECT_EQ(mir_surface_state_fullscreen, surf.state());
+}

